# coding: utf-8

# flake8: noqa

"""
ME TT DataPortal Data Portal API

ME TT DataPortal Data Portal APIs to fetch Gut Microbes Genomes / Genes information.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

__version__ = "0.1.0"

# Define package exports
__all__ = [
    "DrugsApi",
    "EssentialityApi",
    "FitnessApi",
    "GeneFitnessCorrelationsApi",
    "GenesApi",
    "GenomesApi",
    "MutantGrowthApi",
    "OperonsApi",
    "OrthologsApi",
    "PooledTTPInteractionsApi",
    "ProteinProteinInteractionsApi",
    "ProteomicsApi",
    "PyHMMERResultsApi",
    "PyHMMERSearchApi",
    "ReactionsApi",
    "SpeciesApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "AMRSchema",
    "ContigSchema",
    "DBXRefSchema",
    "DrugMICDataSchema",
    "DrugMICSearchQuerySchema",
    "DrugMetabolismDataSchema",
    "DrugMetabolismSearchQuerySchema",
    "EssentialitySearchQuerySchema",
    "FitnessSearchQuerySchema",
    "GeneAdvancedSearchQuerySchema",
    "GenePaginatedResponseSchema",
    "GeneResponseSchema",
    "GeneSearchQuerySchema",
    "GenesByGenomeQuerySchema",
    "GenomePaginatedResponseSchema",
    "GenomeResponseSchema",
    "GenomeSearchQuerySchema",
    "GenomesByIsolateNamesQuerySchema",
    "GetAllGenesQuerySchema",
    "GetAllGenomesQuerySchema",
    "MutantGrowthSearchQuerySchema",
    "PPIAllNeighborsResponseSchema",
    "PPIAllNeighborsSchema",
    "PPIInteractionSchema",
    "PPINeighborsQuerySchema",
    "PPINetworkPropertiesQuerySchema",
    "PPINetworkPropertiesResponseSchema",
    "PPINetworkPropertiesSchema",
    "PPINetworkQuerySchema",
    "PPINetworkResponseSchema",
    "PPINetworkSchema",
    "PPIScoreTypesResponseSchema",
    "PPISearchQuerySchema",
    "PPISearchResponseSchema",
    "PaginatedResponseSchema",
    "PaginatedStrainDrugMICResponseSchema",
    "PaginatedStrainDrugMetabolismResponseSchema",
    "PaginationMetadataSchema",
    "ProteomicsSearchQuerySchema",
    "ReactionsSearchQuerySchema",
    "ResponseStatus",
    "ResultQuerySchema",
    "SearchRequestSchema",
    "SpeciesGenomeSearchQuerySchema",
    "StrainDrugDataResponseSchema",
    "SuccessResponseSchema",
    "TTPCompoundInteractionsQuerySchema",
    "TTPGeneInteractionsQuerySchema",
    "TTPInteractionQuerySchema",
]

# import apis into sdk package
from mett_dataportal_sdk.api.drugs_api import DrugsApi as DrugsApi
from mett_dataportal_sdk.api.essentiality_api import EssentialityApi as EssentialityApi
from mett_dataportal_sdk.api.fitness_api import FitnessApi as FitnessApi
from mett_dataportal_sdk.api.gene_fitness_correlations_api import (
    GeneFitnessCorrelationsApi as GeneFitnessCorrelationsApi,
)
from mett_dataportal_sdk.api.genes_api import GenesApi as GenesApi
from mett_dataportal_sdk.api.genomes_api import GenomesApi as GenomesApi
from mett_dataportal_sdk.api.mutant_growth_api import MutantGrowthApi as MutantGrowthApi
from mett_dataportal_sdk.api.operons_api import OperonsApi as OperonsApi
from mett_dataportal_sdk.api.orthologs_api import OrthologsApi as OrthologsApi
from mett_dataportal_sdk.api.pooled_ttp_interactions_api import (
    PooledTTPInteractionsApi as PooledTTPInteractionsApi,
)
from mett_dataportal_sdk.api.protein_protein_interactions_api import (
    ProteinProteinInteractionsApi as ProteinProteinInteractionsApi,
)
from mett_dataportal_sdk.api.proteomics_api import ProteomicsApi as ProteomicsApi
from mett_dataportal_sdk.api.py_hmmer_results_api import (
    PyHMMERResultsApi as PyHMMERResultsApi,
)
from mett_dataportal_sdk.api.py_hmmer_search_api import (
    PyHMMERSearchApi as PyHMMERSearchApi,
)
from mett_dataportal_sdk.api.reactions_api import ReactionsApi as ReactionsApi
from mett_dataportal_sdk.api.species_api import SpeciesApi as SpeciesApi

# import ApiClient
from mett_dataportal_sdk.api_response import ApiResponse as ApiResponse
from mett_dataportal_sdk.api_client import ApiClient as ApiClient
from mett_dataportal_sdk.configuration import Configuration as Configuration
from mett_dataportal_sdk.exceptions import OpenApiException as OpenApiException
from mett_dataportal_sdk.exceptions import ApiTypeError as ApiTypeError
from mett_dataportal_sdk.exceptions import ApiValueError as ApiValueError
from mett_dataportal_sdk.exceptions import ApiKeyError as ApiKeyError
from mett_dataportal_sdk.exceptions import ApiAttributeError as ApiAttributeError
from mett_dataportal_sdk.exceptions import ApiException as ApiException

# import models into sdk package
from mett_dataportal_sdk.models.amr_schema import AMRSchema as AMRSchema
from mett_dataportal_sdk.models.contig_schema import ContigSchema as ContigSchema
from mett_dataportal_sdk.models.dbx_ref_schema import DBXRefSchema as DBXRefSchema
from mett_dataportal_sdk.models.drug_mic_data_schema import (
    DrugMICDataSchema as DrugMICDataSchema,
)
from mett_dataportal_sdk.models.drug_mic_search_query_schema import (
    DrugMICSearchQuerySchema as DrugMICSearchQuerySchema,
)
from mett_dataportal_sdk.models.drug_metabolism_data_schema import (
    DrugMetabolismDataSchema as DrugMetabolismDataSchema,
)
from mett_dataportal_sdk.models.drug_metabolism_search_query_schema import (
    DrugMetabolismSearchQuerySchema as DrugMetabolismSearchQuerySchema,
)
from mett_dataportal_sdk.models.essentiality_search_query_schema import (
    EssentialitySearchQuerySchema as EssentialitySearchQuerySchema,
)
from mett_dataportal_sdk.models.fitness_search_query_schema import (
    FitnessSearchQuerySchema as FitnessSearchQuerySchema,
)
from mett_dataportal_sdk.models.gene_advanced_search_query_schema import (
    GeneAdvancedSearchQuerySchema as GeneAdvancedSearchQuerySchema,
)
from mett_dataportal_sdk.models.gene_paginated_response_schema import (
    GenePaginatedResponseSchema as GenePaginatedResponseSchema,
)
from mett_dataportal_sdk.models.gene_response_schema import (
    GeneResponseSchema as GeneResponseSchema,
)
from mett_dataportal_sdk.models.gene_search_query_schema import (
    GeneSearchQuerySchema as GeneSearchQuerySchema,
)
from mett_dataportal_sdk.models.genes_by_genome_query_schema import (
    GenesByGenomeQuerySchema as GenesByGenomeQuerySchema,
)
from mett_dataportal_sdk.models.genome_paginated_response_schema import (
    GenomePaginatedResponseSchema as GenomePaginatedResponseSchema,
)
from mett_dataportal_sdk.models.genome_response_schema import (
    GenomeResponseSchema as GenomeResponseSchema,
)
from mett_dataportal_sdk.models.genome_search_query_schema import (
    GenomeSearchQuerySchema as GenomeSearchQuerySchema,
)
from mett_dataportal_sdk.models.genomes_by_isolate_names_query_schema import (
    GenomesByIsolateNamesQuerySchema as GenomesByIsolateNamesQuerySchema,
)
from mett_dataportal_sdk.models.get_all_genes_query_schema import (
    GetAllGenesQuerySchema as GetAllGenesQuerySchema,
)
from mett_dataportal_sdk.models.get_all_genomes_query_schema import (
    GetAllGenomesQuerySchema as GetAllGenomesQuerySchema,
)
from mett_dataportal_sdk.models.mutant_growth_search_query_schema import (
    MutantGrowthSearchQuerySchema as MutantGrowthSearchQuerySchema,
)
from mett_dataportal_sdk.models.ppi_all_neighbors_response_schema import (
    PPIAllNeighborsResponseSchema as PPIAllNeighborsResponseSchema,
)
from mett_dataportal_sdk.models.ppi_all_neighbors_schema import (
    PPIAllNeighborsSchema as PPIAllNeighborsSchema,
)
from mett_dataportal_sdk.models.ppi_interaction_schema import (
    PPIInteractionSchema as PPIInteractionSchema,
)
from mett_dataportal_sdk.models.ppi_neighbors_query_schema import (
    PPINeighborsQuerySchema as PPINeighborsQuerySchema,
)
from mett_dataportal_sdk.models.ppi_network_properties_query_schema import (
    PPINetworkPropertiesQuerySchema as PPINetworkPropertiesQuerySchema,
)
from mett_dataportal_sdk.models.ppi_network_properties_response_schema import (
    PPINetworkPropertiesResponseSchema as PPINetworkPropertiesResponseSchema,
)
from mett_dataportal_sdk.models.ppi_network_properties_schema import (
    PPINetworkPropertiesSchema as PPINetworkPropertiesSchema,
)
from mett_dataportal_sdk.models.ppi_network_query_schema import (
    PPINetworkQuerySchema as PPINetworkQuerySchema,
)
from mett_dataportal_sdk.models.ppi_network_response_schema import (
    PPINetworkResponseSchema as PPINetworkResponseSchema,
)
from mett_dataportal_sdk.models.ppi_network_schema import (
    PPINetworkSchema as PPINetworkSchema,
)
from mett_dataportal_sdk.models.ppi_score_types_response_schema import (
    PPIScoreTypesResponseSchema as PPIScoreTypesResponseSchema,
)
from mett_dataportal_sdk.models.ppi_search_query_schema import (
    PPISearchQuerySchema as PPISearchQuerySchema,
)
from mett_dataportal_sdk.models.ppi_search_response_schema import (
    PPISearchResponseSchema as PPISearchResponseSchema,
)
from mett_dataportal_sdk.models.paginated_response_schema import (
    PaginatedResponseSchema as PaginatedResponseSchema,
)
from mett_dataportal_sdk.models.paginated_strain_drug_mic_response_schema import (
    PaginatedStrainDrugMICResponseSchema as PaginatedStrainDrugMICResponseSchema,
)
from mett_dataportal_sdk.models.paginated_strain_drug_metabolism_response_schema import (
    PaginatedStrainDrugMetabolismResponseSchema as PaginatedStrainDrugMetabolismResponseSchema,
)
from mett_dataportal_sdk.models.pagination_metadata_schema import (
    PaginationMetadataSchema as PaginationMetadataSchema,
)
from mett_dataportal_sdk.models.proteomics_search_query_schema import (
    ProteomicsSearchQuerySchema as ProteomicsSearchQuerySchema,
)
from mett_dataportal_sdk.models.reactions_search_query_schema import (
    ReactionsSearchQuerySchema as ReactionsSearchQuerySchema,
)
from mett_dataportal_sdk.models.response_status import ResponseStatus as ResponseStatus
from mett_dataportal_sdk.models.result_query_schema import (
    ResultQuerySchema as ResultQuerySchema,
)
from mett_dataportal_sdk.models.search_request_schema import (
    SearchRequestSchema as SearchRequestSchema,
)
from mett_dataportal_sdk.models.species_genome_search_query_schema import (
    SpeciesGenomeSearchQuerySchema as SpeciesGenomeSearchQuerySchema,
)
from mett_dataportal_sdk.models.strain_drug_data_response_schema import (
    StrainDrugDataResponseSchema as StrainDrugDataResponseSchema,
)
from mett_dataportal_sdk.models.success_response_schema import (
    SuccessResponseSchema as SuccessResponseSchema,
)
from mett_dataportal_sdk.models.ttp_compound_interactions_query_schema import (
    TTPCompoundInteractionsQuerySchema as TTPCompoundInteractionsQuerySchema,
)
from mett_dataportal_sdk.models.ttp_gene_interactions_query_schema import (
    TTPGeneInteractionsQuerySchema as TTPGeneInteractionsQuerySchema,
)
from mett_dataportal_sdk.models.ttp_interaction_query_schema import (
    TTPInteractionQuerySchema as TTPInteractionQuerySchema,
)



# coding: utf-8

"""
ME TT DataPortal Data Portal API

ME TT DataPortal Data Portal APIs to fetch Gut Microbes Genomes / Genes information.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from mett_dataportal_sdk.models.drug_metabolism_data_schema import (
    DrugMetabolismDataSchema,
)
from mett_dataportal_sdk.models.drug_mic_data_schema import DrugMICDataSchema
from typing import Set
from typing_extensions import Self


class StrainDrugDataResponseSchema(BaseModel):
    """
    Response schema for combined strain drug data (MIC + metabolism).
    """  # noqa: E501

    isolate_name: StrictStr = Field(description="Strain isolate name")
    species_acronym: Optional[StrictStr] = None
    species_scientific_name: Optional[StrictStr] = None
    drug_mic_data: Optional[List[DrugMICDataSchema]] = Field(
        default=None, description="List of drug MIC measurements"
    )
    drug_metabolism_data: Optional[List[DrugMetabolismDataSchema]] = Field(
        default=None, description="List of drug metabolism measurements"
    )
    __properties: ClassVar[List[str]] = [
        "isolate_name",
        "species_acronym",
        "species_scientific_name",
        "drug_mic_data",
        "drug_metabolism_data",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of StrainDrugDataResponseSchema from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in drug_mic_data (list)
        _items = []
        if self.drug_mic_data:
            for _item_drug_mic_data in self.drug_mic_data:
                if _item_drug_mic_data:
                    _items.append(_item_drug_mic_data.to_dict())
            _dict["drug_mic_data"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in drug_metabolism_data (list)
        _items = []
        if self.drug_metabolism_data:
            for _item_drug_metabolism_data in self.drug_metabolism_data:
                if _item_drug_metabolism_data:
                    _items.append(_item_drug_metabolism_data.to_dict())
            _dict["drug_metabolism_data"] = _items
        # set to None if species_acronym (nullable) is None
        # and model_fields_set contains the field
        if self.species_acronym is None and "species_acronym" in self.model_fields_set:
            _dict["species_acronym"] = None

        # set to None if species_scientific_name (nullable) is None
        # and model_fields_set contains the field
        if (
            self.species_scientific_name is None
            and "species_scientific_name" in self.model_fields_set
        ):
            _dict["species_scientific_name"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of StrainDrugDataResponseSchema from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "isolate_name": obj.get("isolate_name"),
                "species_acronym": obj.get("species_acronym"),
                "species_scientific_name": obj.get("species_scientific_name"),
                "drug_mic_data": [
                    DrugMICDataSchema.from_dict(_item) for _item in obj["drug_mic_data"]
                ]
                if obj.get("drug_mic_data") is not None
                else None,
                "drug_metabolism_data": [
                    DrugMetabolismDataSchema.from_dict(_item)
                    for _item in obj["drug_metabolism_data"]
                ]
                if obj.get("drug_metabolism_data") is not None
                else None,
            }
        )
        return _obj



"""Gene CLI commands."""

from __future__ import annotations

from typing import List, Optional

import typer  # type: ignore[import]

from ..utils import comma_join, ensure_client, handle_raw_response, merge_params

genes_app = typer.Typer(help="Gene endpoints")


@genes_app.command("list")
def genes_list(
    ctx: typer.Context,
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    sort_field: Optional[str] = typer.Option(None, "--sort-field"),
    sort_order: Optional[str] = typer.Option(None, "--sort-order"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "page": page,
            "per_page": per_page,
            "sort_field": sort_field,
            "sort_order": sort_order,
        }
    )
    response = client.raw_request("GET", "/api/genes/", params=params, format=format)
    handle_raw_response(response, format, title="Genes")


@genes_app.command("search")
def genes_search(
    ctx: typer.Context,
    query: Optional[str] = typer.Option(None, "--query", "-q"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    sort_field: Optional[str] = typer.Option(None, "--sort-field"),
    sort_order: Optional[str] = typer.Option(None, "--sort-order"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "query": query,
            "page": page,
            "per_page": per_page,
            "sort_field": sort_field,
            "sort_order": sort_order,
        }
    )
    response = client.raw_request(
        "GET", "/api/genes/search", params=params, format=format
    )
    handle_raw_response(response, format, title="Gene search")


@genes_app.command("search-advanced")
def genes_search_advanced(
    ctx: typer.Context,
    isolates: Optional[List[str]] = typer.Option(None, "--isolate", "-i"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    locus_tag: Optional[str] = typer.Option(None, "--locus-tag"),
    query: Optional[str] = typer.Option(None, "--query", "-q"),
    filter: Optional[str] = typer.Option(None, "--filter"),
    filter_operators: Optional[str] = typer.Option(None, "--filter-operators"),
    seq_id: Optional[str] = typer.Option(None, "--seq-id"),
    start_position: Optional[int] = typer.Option(None, "--start-position"),
    end_position: Optional[int] = typer.Option(None, "--end-position"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    sort_field: Optional[str] = typer.Option(None, "--sort-field"),
    sort_order: Optional[str] = typer.Option(None, "--sort-order"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "isolates": comma_join(isolates),
            "species_acronym": species_acronym,
            "locus_tag": locus_tag,
            "query": query,
            "filter": filter,
            "filter_operators": filter_operators,
            "seq_id": seq_id,
            "start_position": start_position,
            "end_position": end_position,
            "page": page,
            "per_page": per_page,
            "sort_field": sort_field,
            "sort_order": sort_order,
        }
    )
    response = client.raw_request(
        "GET", "/api/genes/search/advanced", params=params, format=format
    )
    handle_raw_response(response, format, title="Advanced gene search")


@genes_app.command("get")
def genes_get(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(..., help="Locus tag"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request("GET", f"/api/genes/{locus_tag}", format=format)
    handle_raw_response(response, format, title=f"Gene {locus_tag}")


@genes_app.command("proteomics")
def genes_proteomics(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/genes/{locus_tag}/proteomics", format=format
    )
    handle_raw_response(response, format, title=f"Proteomics ({locus_tag})")


@genes_app.command("essentiality")
def genes_essentiality(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/genes/{locus_tag}/essentiality", format=format
    )
    handle_raw_response(response, format, title=f"Essentiality ({locus_tag})")


@genes_app.command("fitness")
def genes_fitness(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/genes/{locus_tag}/fitness", format=format
    )
    handle_raw_response(response, format, title=f"Fitness ({locus_tag})")


@genes_app.command("mutant-growth")
def genes_mutant_growth(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/genes/{locus_tag}/mutant-growth", format=format
    )
    handle_raw_response(response, format, title=f"Mutant growth ({locus_tag})")


@genes_app.command("reactions")
def genes_reactions(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/genes/{locus_tag}/reactions", format=format
    )
    handle_raw_response(response, format, title=f"Reactions ({locus_tag})")


@genes_app.command("correlations")
def genes_correlations(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    min_correlation: Optional[float] = typer.Option(None, "--min"),
    max_results: Optional[int] = typer.Option(None, "--max-results"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "min_correlation": min_correlation,
            "max_results": max_results,
        }
    )
    response = client.raw_request(
        "GET",
        f"/api/genes/{locus_tag}/correlations",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Correlations ({locus_tag})")


@genes_app.command("orthologs")
def genes_orthologs(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    orthology_type: Optional[str] = typer.Option(None, "--orthology-type"),
    one_to_one_only: Optional[bool] = typer.Option(None, "--one-to-one-only"),
    cross_species_only: Optional[bool] = typer.Option(None, "--cross-species-only"),
    max_results: Optional[int] = typer.Option(None, "--max-results"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "orthology_type": orthology_type,
            "one_to_one_only": one_to_one_only,
            "cross_species_only": cross_species_only,
            "max_results": max_results,
        }
    )
    response = client.raw_request(
        "GET",
        f"/api/genes/{locus_tag}/orthologs",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Orthologs ({locus_tag})")


@genes_app.command("operons")
def genes_operons(
    ctx: typer.Context,
    locus_tag: str = typer.Argument(...),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params({"species_acronym": species_acronym})
    response = client.raw_request(
        "GET",
        f"/api/genes/{locus_tag}/operons",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Operons ({locus_tag})")


@genes_app.command("autocomplete")
def genes_autocomplete(
    ctx: typer.Context,
    query: str = typer.Option(..., "--query", "-q"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    isolates: Optional[List[str]] = typer.Option(None, "--isolates", "-i"),
    filter: Optional[str] = typer.Option(
        None, "--filter", help="Filter expression (e.g., 'essentiality:essential')"
    ),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "query": query,
            "species_acronym": species_acronym,
            "isolates": comma_join(isolates),
            "filter": filter,
            "page": page,
            "per_page": per_page,
        }
    )
    response = client.raw_request(
        "GET", "/api/genes/autocomplete", params=params, format=format
    )
    handle_raw_response(response, format, title="Gene autocomplete")


@genes_app.command("faceted-search")
def genes_faceted_search(
    ctx: typer.Context,
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    limit: Optional[int] = typer.Option(None, "--limit"),
    pfam: Optional[str] = typer.Option(None, "--pfam"),
    interpro: Optional[str] = typer.Option(None, "--interpro"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "limit": limit,
            "pfam": pfam,
            "interpro": interpro,
        }
    )
    response = client.raw_request(
        "GET", "/api/genes/faceted-search", params=params, format=format
    )
    handle_raw_response(response, format, title="Gene facets")


@genes_app.command("protein")
def genes_protein(
    ctx: typer.Context,
    protein_id: str = typer.Argument(..., help="Protein ID"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/genes/protein/{protein_id}", format=format
    )
    handle_raw_response(response, format, title=f"Protein {protein_id}")



"""Orthologs CLI commands."""

from __future__ import annotations

from typing import Optional

import typer  # type: ignore[import]

from ..utils import ensure_client, handle_raw_response, merge_params

orthologs_app = typer.Typer(help="Ortholog endpoints")


@orthologs_app.command("search")
def orthologs_search(
    ctx: typer.Context,
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    orthology_type: Optional[str] = typer.Option(None, "--orthology-type"),
    one_to_one_only: Optional[bool] = typer.Option(None, "--one-to-one-only"),
    cross_species_only: Optional[bool] = typer.Option(None, "--cross-species-only"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "orthology_type": orthology_type,
            "one_to_one_only": one_to_one_only,
            "cross_species_only": cross_species_only,
            "page": page,
            "per_page": per_page,
        }
    )
    response = client.raw_request(
        "GET", "/api/orthologs/search", params=params, format=format
    )
    handle_raw_response(response, format, title="Ortholog search")


@orthologs_app.command("pair")
def orthologs_pair(
    ctx: typer.Context,
    locus_tag_a: str = typer.Option(..., "--gene-a"),
    locus_tag_b: str = typer.Option(..., "--gene-b"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = {
        "locus_tag_a": locus_tag_a,
        "locus_tag_b": locus_tag_b,
    }
    response = client.raw_request(
        "GET", "/api/orthologs/pair", params=params, format=format
    )
    handle_raw_response(response, format, title="Ortholog pair")



from __future__ import annotations

from click.testing import CliRunner
from typer.main import get_command

from mett_client.cli.main import app as cli_app
from .test_cli import _patch_dummy_client


runner = CliRunner()
cli_cmd = get_command(cli_app)


def test_genes_list(monkeypatch) -> None:
    """Friendly CLI: mett genes list --page 1 --per-page 10 --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd,
        ["genes", "list", "--page", "1", "--per-page", "10", "--format", "json"],
    )
    assert result.exit_code == 0


def test_genes_search_basic(monkeypatch) -> None:
    """Friendly CLI: mett genes search --query dnaA --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd, ["genes", "search", "--query", "dnaA", "--format", "json"]
    )
    assert result.exit_code == 0


def test_genes_faceted_search_species_pfam(monkeypatch) -> None:
    """Friendly CLI: mett genes faceted-search --species BU --limit 10 --pfam PF07660,PF07715 --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd,
        [
            "genes",
            "faceted-search",
            "--species",
            "BU",
            "--limit",
            "10",
            "--pfam",
            "PF07660,PF07715",
            "--format",
            "json",
        ],
    )
    assert result.exit_code == 0


def test_genes_autocomplete_species_query(monkeypatch) -> None:
    """Friendly CLI: mett genes autocomplete --species BU --query dnaA --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd,
        [
            "genes",
            "autocomplete",
            "--species",
            "BU",
            "--query",
            "dnaA",
            "--format",
            "json",
        ],
    )
    assert result.exit_code == 0


def test_genes_search_advanced_locus_tag(monkeypatch) -> None:
    """Friendly CLI: mett genes search-advanced --locus-tag BU_ATCC8492_00001 --per-page 1 --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd,
        [
            "genes",
            "search-advanced",
            "--locus-tag",
            "BU_ATCC8492_00001",
            "--per-page",
            "1",
            "--format",
            "json",
        ],
    )
    assert result.exit_code == 0



"""Convenience re-exports of generated SDK models."""

from __future__ import annotations

from typing import TypedDict

from mett_dataportal_sdk.models.drug_metabolism_data_schema import (
    DrugMetabolismDataSchema,
)
from mett_dataportal_sdk.models.drug_mic_data_schema import DrugMICDataSchema
from mett_dataportal_sdk.models.gene_paginated_response_schema import (
    GenePaginatedResponseSchema,
)
from mett_dataportal_sdk.models.gene_response_schema import GeneResponseSchema
from mett_dataportal_sdk.models.genome_paginated_response_schema import (
    GenomePaginatedResponseSchema,
)
from mett_dataportal_sdk.models.genome_response_schema import GenomeResponseSchema
from mett_dataportal_sdk.models.paginated_response_schema import PaginatedResponseSchema
from mett_dataportal_sdk.models.paginated_strain_drug_metabolism_response_schema import (
    PaginatedStrainDrugMetabolismResponseSchema,
)
from mett_dataportal_sdk.models.paginated_strain_drug_mic_response_schema import (
    PaginatedStrainDrugMICResponseSchema,
)
from mett_dataportal_sdk.models.pagination_metadata_schema import (
    PaginationMetadataSchema,
)
from mett_dataportal_sdk.models.success_response_schema import SuccessResponseSchema


class SpeciesDict(TypedDict, total=False):
    """Lightweight species representation."""

    species_acronym: str
    species_scientific_name: str
    description: str
    taxonomy_id: int


Pagination = PaginationMetadataSchema
PaginatedResponse = PaginatedResponseSchema
GenomePage = GenomePaginatedResponseSchema
GenePage = GenePaginatedResponseSchema
StrainDrugMICPage = PaginatedStrainDrugMICResponseSchema
StrainDrugMetabolismPage = PaginatedStrainDrugMetabolismResponseSchema
SuccessResponse = SuccessResponseSchema

Genome = GenomeResponseSchema
Gene = GeneResponseSchema
DrugMIC = DrugMICDataSchema
DrugMetabolism = DrugMetabolismDataSchema
Species = SpeciesDict

__all__ = [
    "Pagination",
    "PaginatedResponse",
    "GenomePage",
    "GenePage",
    "StrainDrugMICPage",
    "StrainDrugMetabolismPage",
    "SuccessResponse",
    "Genome",
    "Gene",
    "DrugMIC",
    "DrugMetabolism",
    "Species",
]



"""Drug CLI commands."""

from __future__ import annotations

from typing import Optional

import typer  # type: ignore[import]

from ..output import print_full_table, print_json, print_tsv
from ..utils import ensure_client, handle_raw_response, merge_params

drugs_app = typer.Typer(help="Drug endpoints")


@drugs_app.command("mic")
def drug_mic_search(
    ctx: typer.Context,
    query: Optional[str] = typer.Option(None, "--query", "-q"),
    drug_name: Optional[str] = typer.Option(None, "--drug-name"),
    drug_class: Optional[str] = typer.Option(None, "--drug-class"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    min_mic_value: Optional[float] = typer.Option(None, "--min-mic-value"),
    max_mic_value: Optional[float] = typer.Option(None, "--max-mic-value"),
    unit: Optional[str] = typer.Option(None, "--unit"),
    experimental_condition: Optional[str] = typer.Option(None, "--condition"),
    page: int = typer.Option(1, "--page", "-p"),
    per_page: int = typer.Option(20, "--per-page"),
    sort_by: Optional[str] = typer.Option(None, "--sort-by"),
    sort_order: Optional[str] = typer.Option(None, "--sort-order"),
    format: Optional[str] = typer.Option(
        None, "--format", "-f", help="Output format: json|tsv (default: table)"
    ),
) -> None:
    client = ensure_client(ctx)
    api_format = format or "json"  # Default to JSON for API request
    result = client.search_drug_mic(
        format=api_format,
        query=query,
        drug_name=drug_name,
        drug_class=drug_class,
        species_acronym=species_acronym,
        min_mic_value=min_mic_value,
        max_mic_value=max_mic_value,
        unit=unit,
        experimental_condition=experimental_condition,
        page=page,
        per_page=per_page,
        sort_by=sort_by,
        sort_order=sort_order,
    )

    if format == "tsv":
        print_tsv(result.items)
    elif format == "json":
        print_json(result.raw)
    else:
        # No format specified - display as table
        print_full_table(result.items, title="Drug MIC")


@drugs_app.command("mic-by-drug")
def drug_mic_by_drug(
    ctx: typer.Context,
    drug_name: str = typer.Argument(..., help="Drug name"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params({"species_acronym": species_acronym})
    response = client.raw_request(
        "GET",
        f"/api/drugs/mic/by-drug/{drug_name}",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Drug MIC ({drug_name})")


@drugs_app.command("metabolism-search")
def drug_metabolism_search(
    ctx: typer.Context,
    query: Optional[str] = typer.Option(None, "--query", "-q"),
    drug_name: Optional[str] = typer.Option(None, "--drug-name"),
    drug_class: Optional[str] = typer.Option(None, "--drug-class"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    min_fdr: Optional[float] = typer.Option(None, "--min-fdr"),
    min_degr_percent: Optional[float] = typer.Option(None, "--min-degr-percent"),
    metabolizer_classification: Optional[str] = typer.Option(None, "--classification"),
    is_significant: Optional[bool] = typer.Option(None, "--significant"),
    experimental_condition: Optional[str] = typer.Option(None, "--condition"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    sort_by: Optional[str] = typer.Option(None, "--sort-by"),
    sort_order: Optional[str] = typer.Option(None, "--sort-order"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "query": query,
            "drug_name": drug_name,
            "drug_class": drug_class,
            "species_acronym": species_acronym,
            "min_fdr": min_fdr,
            "min_degr_percent": min_degr_percent,
            "metabolizer_classification": metabolizer_classification,
            "is_significant": is_significant,
            "experimental_condition": experimental_condition,
            "page": page,
            "per_page": per_page,
            "sort_by": sort_by,
            "sort_order": sort_order,
        }
    )
    response = client.raw_request(
        "GET", "/api/drugs/metabolism/search", params=params, format=format
    )
    handle_raw_response(response, format, title="Drug metabolism search")


@drugs_app.command("metabolism-by-drug")
def drug_metabolism_by_drug(
    ctx: typer.Context,
    drug_name: str = typer.Argument(..., help="Drug name"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params({"species_acronym": species_acronym})
    response = client.raw_request(
        "GET",
        f"/api/drugs/metabolism/by-drug/{drug_name}",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Drug metabolism ({drug_name})")


@drugs_app.command("mic-by-class")
def drug_mic_by_class(
    ctx: typer.Context,
    drug_class: str = typer.Argument(...),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "page": page,
            "per_page": per_page,
        }
    )
    response = client.raw_request(
        "GET",
        f"/api/drugs/mic/by-class/{drug_class}",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Drug MIC class ({drug_class})")


@drugs_app.command("metabolism-by-class")
def drug_metabolism_by_class(
    ctx: typer.Context,
    drug_class: str = typer.Argument(...),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "page": page,
            "per_page": per_page,
        }
    )
    response = client.raw_request(
        "GET",
        f"/api/drugs/metabolism/by-class/{drug_class}",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"Drug metabolism class ({drug_class})")



# flake8: noqa

# import apis into api package
from mett_dataportal_sdk.api.drugs_api import DrugsApi
from mett_dataportal_sdk.api.essentiality_api import EssentialityApi
from mett_dataportal_sdk.api.fitness_api import FitnessApi
from mett_dataportal_sdk.api.gene_fitness_correlations_api import (
    GeneFitnessCorrelationsApi,
)
from mett_dataportal_sdk.api.genes_api import GenesApi
from mett_dataportal_sdk.api.genomes_api import GenomesApi
from mett_dataportal_sdk.api.mutant_growth_api import MutantGrowthApi
from mett_dataportal_sdk.api.operons_api import OperonsApi
from mett_dataportal_sdk.api.orthologs_api import OrthologsApi
from mett_dataportal_sdk.api.pooled_ttp_interactions_api import PooledTTPInteractionsApi
from mett_dataportal_sdk.api.protein_protein_interactions_api import (
    ProteinProteinInteractionsApi,
)
from mett_dataportal_sdk.api.proteomics_api import ProteomicsApi
from mett_dataportal_sdk.api.py_hmmer_results_api import PyHMMERResultsApi
from mett_dataportal_sdk.api.py_hmmer_search_api import PyHMMERSearchApi
from mett_dataportal_sdk.api.reactions_api import ReactionsApi
from mett_dataportal_sdk.api.species_api import SpeciesApi



# coding: utf-8

"""
ME TT DataPortal Data Portal API

ME TT DataPortal Data Portal APIs to fetch Gut Microbes Genomes / Genes information.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Set
from typing_extensions import Self


class EssentialitySearchQuerySchema(BaseModel):
    """
    Schema for essentiality search endpoint query parameters.
    """  # noqa: E501

    locus_tags: Optional[StrictStr] = None
    uniprot_ids: Optional[StrictStr] = None
    essentiality_call: Optional[StrictStr] = None
    experimental_condition: Optional[StrictStr] = None
    min_tas_in_locus: Optional[Annotated[int, Field(strict=True, ge=0)]] = None
    min_tas_hit: Optional[
        Union[
            Annotated[float, Field(le=1, strict=True, ge=0)],
            Annotated[int, Field(le=1, strict=True, ge=0)],
        ]
    ] = None
    element: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "locus_tags",
        "uniprot_ids",
        "essentiality_call",
        "experimental_condition",
        "min_tas_in_locus",
        "min_tas_hit",
        "element",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of EssentialitySearchQuerySchema from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if locus_tags (nullable) is None
        # and model_fields_set contains the field
        if self.locus_tags is None and "locus_tags" in self.model_fields_set:
            _dict["locus_tags"] = None

        # set to None if uniprot_ids (nullable) is None
        # and model_fields_set contains the field
        if self.uniprot_ids is None and "uniprot_ids" in self.model_fields_set:
            _dict["uniprot_ids"] = None

        # set to None if essentiality_call (nullable) is None
        # and model_fields_set contains the field
        if (
            self.essentiality_call is None
            and "essentiality_call" in self.model_fields_set
        ):
            _dict["essentiality_call"] = None

        # set to None if experimental_condition (nullable) is None
        # and model_fields_set contains the field
        if (
            self.experimental_condition is None
            and "experimental_condition" in self.model_fields_set
        ):
            _dict["experimental_condition"] = None

        # set to None if min_tas_in_locus (nullable) is None
        # and model_fields_set contains the field
        if (
            self.min_tas_in_locus is None
            and "min_tas_in_locus" in self.model_fields_set
        ):
            _dict["min_tas_in_locus"] = None

        # set to None if min_tas_hit (nullable) is None
        # and model_fields_set contains the field
        if self.min_tas_hit is None and "min_tas_hit" in self.model_fields_set:
            _dict["min_tas_hit"] = None

        # set to None if element (nullable) is None
        # and model_fields_set contains the field
        if self.element is None and "element" in self.model_fields_set:
            _dict["element"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of EssentialitySearchQuerySchema from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "locus_tags": obj.get("locus_tags"),
                "uniprot_ids": obj.get("uniprot_ids"),
                "essentiality_call": obj.get("essentiality_call"),
                "experimental_condition": obj.get("experimental_condition"),
                "min_tas_in_locus": obj.get("min_tas_in_locus"),
                "min_tas_hit": obj.get("min_tas_hit"),
                "element": obj.get("element"),
            }
        )
        return _obj



"""PPI (Protein-Protein Interactions) CLI commands."""

from __future__ import annotations

from typing import Optional

import typer  # type: ignore[import]

from ..utils import ensure_client, handle_raw_response, merge_params

ppi_app = typer.Typer(help="PPI endpoints")


@ppi_app.command("scores")
def ppi_scores(
    ctx: typer.Context,
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request("GET", "/api/ppi/scores/available", format=format)
    handle_raw_response(response, format, title="PPI score types")


@ppi_app.command("interactions")
def ppi_interactions(
    ctx: typer.Context,
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    isolate_name: Optional[str] = typer.Option(None, "--isolate"),
    score_type: Optional[str] = typer.Option(None, "--score-type"),
    score_threshold: Optional[float] = typer.Option(None, "--score-threshold"),
    has_xlms: Optional[bool] = typer.Option(None, "--has-xlms"),
    has_string: Optional[bool] = typer.Option(None, "--has-string"),
    has_operon: Optional[bool] = typer.Option(None, "--has-operon"),
    has_ecocyc: Optional[bool] = typer.Option(None, "--has-ecocyc"),
    protein_id: Optional[str] = typer.Option(None, "--protein-id"),
    locus_tag: Optional[str] = typer.Option(None, "--locus-tag"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    per_page: Optional[int] = typer.Option(None, "--per-page"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "species_acronym": species_acronym,
            "isolate_name": isolate_name,
            "score_type": score_type,
            "score_threshold": score_threshold,
            "has_xlms": has_xlms,
            "has_string": has_string,
            "has_operon": has_operon,
            "has_ecocyc": has_ecocyc,
            "protein_id": protein_id,
            "locus_tag": locus_tag,
            "page": page,
            "per_page": per_page,
        }
    )
    response = client.raw_request(
        "GET", "/api/ppi/interactions", params=params, format=format
    )
    handle_raw_response(response, format, title="PPI interactions")


@ppi_app.command("neighbors")
def ppi_neighbors(
    ctx: typer.Context,
    protein_id: Optional[str] = typer.Option(None, "--protein-id"),
    locus_tag: Optional[str] = typer.Option(None, "--locus-tag"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "protein_id": protein_id,
            "locus_tag": locus_tag,
            "species_acronym": species_acronym,
        }
    )
    response = client.raw_request(
        "GET", "/api/ppi/neighbors", params=params, format=format
    )
    handle_raw_response(response, format, title="PPI neighbors")


@ppi_app.command("neighborhood")
def ppi_neighborhood(
    ctx: typer.Context,
    protein_id: Optional[str] = typer.Option(None, "--protein-id"),
    locus_tag: Optional[str] = typer.Option(None, "--locus-tag"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    n: Optional[int] = typer.Option(None, "--n"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "protein_id": protein_id,
            "locus_tag": locus_tag,
            "species_acronym": species_acronym,
            "n": n,
        }
    )
    response = client.raw_request(
        "GET", "/api/ppi/neighborhood", params=params, format=format
    )
    handle_raw_response(response, format, title="PPI neighborhood")


@ppi_app.command("network")
def ppi_network(
    ctx: typer.Context,
    score_type: str = typer.Argument(...),
    score_threshold: Optional[float] = typer.Option(None, "--score-threshold"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    include_properties: Optional[bool] = typer.Option(None, "--include-properties"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "score_threshold": score_threshold,
            "species_acronym": species_acronym,
            "include_properties": include_properties,
        }
    )
    response = client.raw_request(
        "GET",
        f"/api/ppi/network/{score_type}",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"PPI network ({score_type})")


@ppi_app.command("network-properties")
def ppi_network_properties(
    ctx: typer.Context,
    score_type: str = typer.Option(..., "--score-type"),
    score_threshold: Optional[float] = typer.Option(None, "--score-threshold"),
    species_acronym: Optional[str] = typer.Option(None, "--species", "-s"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "score_type": score_type,
            "score_threshold": score_threshold,
            "species_acronym": species_acronym,
        }
    )
    response = client.raw_request(
        "GET", "/api/ppi/network-properties", params=params, format=format
    )
    handle_raw_response(response, format, title="PPI network properties")



"""Shared utilities and cross-cutting tests for CLI testing.

This module provides:
- DummyClient, DummyResponse, DummyResult: Mock objects for testing without real API calls
- _patch_dummy_client(): Helper to patch the CLI to use DummyClient
- Cross-cutting tests (system, generic API) that don't fit in domain-specific test files

Domain-specific tests are organized in separate files:
- test_cli_species.py: Species endpoints
- test_cli_genomes.py: Genome endpoints
- test_cli_genes.py: Gene endpoints
- test_cli_experimental.py: Experimental APIs (drugs, PPI, orthologs, operons, fitness, etc.)
"""

from __future__ import annotations

import json
from typing import Any, Callable

from click.testing import CliRunner
from typer.main import get_command

from mett_client.cli.main import app as cli_app


class DummyResponse:
    """Minimal HTTP-like response object for handle_raw_response."""

    def __init__(self, payload: Any | None = None) -> None:
        if payload is None:
            payload = {"ok": True}
        self._payload = payload
        self.headers = {"Content-Type": "application/json"}
        self.text = json.dumps(self._payload)

    def json(self) -> Any:  # pragma: no cover - trivial
        return self._payload


class DummyResult:
    """Result object for high-level client methods (with items/raw)."""

    def __init__(self, payload: Any | None = None) -> None:
        if payload is None:
            payload = [{"ok": True}]
        self.items = payload
        self.raw = {"data": payload}


class DummyClient:
    """Generic stand‑in for DataPortalClient.

    - raw_request(...) -> DummyResponse
    - Any other attribute call (e.g., list_genomes, search_genomes, etc.)
      returns a DummyResult so CLI pagination/printing logic works.
    """

    def raw_request(self, *args: Any, **kwargs: Any) -> DummyResponse:
        return DummyResponse()

    def __getattr__(self, _name: str) -> Callable[..., DummyResult]:
        def _method(*_args: Any, **_kwargs: Any) -> DummyResult:
            return DummyResult()

        return _method


runner = CliRunner()
cli_cmd = get_command(cli_app)


def _patch_dummy_client(monkeypatch) -> None:
    """Patch CLI to always use DummyClient instead of real HTTP client."""

    from mett_client import cli as cli_pkg
    from mett_client.cli import main as main_module
    from mett_client.cli import utils as cli_utils

    dummy = DummyClient()

    # Callback in main.py uses _build_client → patch it
    monkeypatch.setattr(main_module, "_build_client", lambda **_: dummy)

    # Safety: ensure_client(ctx) should also fall back to DummyClient
    def _ensure_client(ctx):
        if ctx.obj is None:
            ctx.obj = dummy
        return ctx.obj

    monkeypatch.setattr(cli_utils, "ensure_client", _ensure_client)

    # Also make the package-level default (if ever used) point to DummyClient
    if hasattr(cli_pkg, "DEFAULT_CLIENT"):
        monkeypatch.setattr(cli_pkg, "DEFAULT_CLIENT", dummy)


# Cross-cutting tests (system, generic API, etc.)
# Domain-specific tests are in test_cli_*.py files


def test_system_health(monkeypatch) -> None:
    """Friendly CLI: mett system health --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(cli_cmd, ["system", "health", "--format", "json"])
    assert result.exit_code == 0


def test_api_request_generic(monkeypatch) -> None:
    """Generic CLI: mett api request GET /api/species/ --format json"""
    _patch_dummy_client(monkeypatch)
    args = ["api", "request", "GET", "/api/species/", "--format", "json"]
    result = runner.invoke(cli_cmd, args)
    assert result.exit_code == 0
